import ipywidgets as widgets
from IPython.display import display
from dbispipeline.analytics import (
    extract_gridsearch_parameters,
)


class FilterWidget:
    def __init__(
            self,
            df,
            keep_columns=None,
            drop_columns=None,
            extract_grid_params=True,
            score_name='mean_test_score',
            auto_open=True,
    ):
        if keep_columns is not None and drop_columns is not None:
            raise ValueError(
                'only use one of either keep_columns or drop_columns')

        if keep_columns:
            kept_columns = keep_columns
        elif drop_columns:
            kept_columns = [c for c in df.columns if c not in drop_columns]
        else:
            kept_columns = df.columns

        self.original_df = df

        self.filter_widgets = {
            c: widgets.Checkbox(description=c, value=c in kept_columns) for c
            in df.columns}
        self.all_button = widgets.Button(description='All')
        self.none_button = widgets.Button(description='None')
        self.invert_button = widgets.Button(description='Invert')
        self.all_button.on_click(self.set_all)
        self.none_button.on_click(self.set_none)
        self.invert_button.on_click(self.set_invert)

        self.extract_grid_params = widgets.Checkbox(
            description='Extract GridSearch Parameters',
            value=extract_grid_params)
        self.score_name = widgets.Text(
            description='Score Name', value=score_name)
        self.filter_button = widgets.Button(description="Filter")
        self.filter_button.on_click(self.do_filter)

        if auto_open:
            display(widgets.VBox([
                widgets.HBox([
                    widgets.VBox([
                        self.all_button,
                        self.none_button,
                        self.invert_button,
                    ]),
                    widgets.VBox(list(self.filter_widgets.values())),
                ]),
                self.extract_grid_params,
                self.score_name,
                self.filter_button,
            ], layout={'border': '1px solid grey'}))

    def set_invert(self, *args):
        self._set(lambda x: not x.value)

    def set_all(self, *args):
        self._set(lambda x: True)

    def set_none(self, *args):
        self._set(lambda x: False)

    def _set(self, callback):
        for w in self.filter_widgets.values():
            w.value = callback(w)

    def do_filter(self, *args):
        selected_columns = [k for k, v in self.filter_widgets.items() if
                            v.value]
        self.df = self.original_df[selected_columns]
        if self.extract_grid_params.value:
            score_names = self.score_name.value.split(',')
            self.df = extract_gridsearch_parameters(self.df, score_names)
            print('extracted grid parameters')


import logging
import os

import ipywidgets as widgets
import matplotlib.pyplot as plt
from IPython.display import display

plt.rcParams['figure.figsize'] = (20, 10)
logger = logging.getLogger(__name__)


def plot_multi_scores(
        df,
        group_keys=None,
        filter_key=None,
        filter_values=None,
        method='max',
        output_file=None,
        transpose=True,
        score_names=None):
    tab_outputs = []
    tab = widgets.Tab()
    for i, score_name in enumerate(score_names):
        output = widgets.Output()
        tab_outputs.append(output)
        tab.set_title(i, score_name)
        with output:
            other_scores = [s for s in score_names if s != score_name]
            sub_df = df.drop(columns=other_scores)
            widget = PlotWidget(sub_df, group_keys, filter_key, filter_values,
                                method,
                                output_file, transpose, score_name)

    tab.children = tab_outputs
    display(tab)


class PlotWidget:
    def __init__(
            self,
            df,
            group_keys=None,
            filter_key=None,
            filter_values=None,
            method='max',
            output_file=None,
            transpose=True,
            score_name=None,
    ):
        if score_name is not None:
            group_columns = [c for c in df.columns if c != score_name]
            df = df.fillna('N/A').groupby(group_columns).max()

        self.df = df
        self.filtered_df = df

        index_names = sorted(df.index.names)

        self.w_parameters = widgets.SelectMultiple(
            options=index_names,
            value=group_keys if group_keys is not None else [index_names[0]],
            description='Group By',
        )
        valid_methods = ['min', 'max', 'mean', 'box']
        self.w_method = widgets.RadioButtons(
            options=valid_methods,
            value=method if method in valid_methods else 'max'
        )
        self.w_transpose = widgets.Checkbox(description='transpose',
                                            value=transpose)
        self.w_filter_key = widgets.Select(
            options=[''] + index_names[:],
            description='Filter field',
            value=filter_key if filter_key is not None else '',
        )
        self.w_filter_value = widgets.SelectMultiple(
            options=[], description='Filter value'
        )
        self.update_possible_filter_values(None)
        if filter_values is not None:
            self.w_filter_value.value = filter_values

        self.w_file_name = widgets.Text(
            value=output_file if output_file is not None else '',
            description='output CSV Path',
        )
        self.w_overwrite = widgets.Checkbox(
            description='allows overwriting output files'
        )
        self.w_output_button = widgets.Button(description='Generate CSV')
        self.w_export_output = widgets.Output()

        self.w_filter_key.observe(self.update_possible_filter_values)
        self.w_output_button.on_click(self.csv_callback)
        self.display()

    def update_possible_filter_values(self, x):
        value = self.w_filter_key.value
        if len(value.strip()):
            self.w_filter_value.options = [''] + sorted(
                self.df.index.unique(level=value)
            )
        else:
            self.w_filter_value.options = []

    def plot_callback(self, parameter, method, filter_key, filter_value,
                      transpose):
        if filter_key and filter_value:
            df = self.df[
                self.df.index.get_level_values(filter_key).isin(filter_value)]
        else:
            df = self.df
        group = df.groupby(list(parameter))
        if method == 'box':
            group.boxplot(subplots=False, rot=90, figsize=(20, 5))
            self.filtered_df = df
            if transpose:
                self.filtered_df = self.filtered_df.T
        else:
            if method == 'min':
                self.filtered_df = group.min()
            elif method == 'max':
                self.filtered_df = group.max()
            elif method == 'mean':
                self.filtered_df = group.mean()
            if transpose:
                self.filtered_df = self.filtered_df.T
            self.filtered_df.plot.bar()

    def csv_callback(self, click_event):
        path = self.w_file_name.value
        if os.path.isfile(path) and not self.w_overwrite.value:
            logger.warn('not overwriting existing file: {path}')
            return
        with self.w_export_output:
            self.w_export_output.clear_output()
            self.filtered_df.reset_index().to_csv(path, index=False)
            print(f'wrote output to {path}')

    def get_output(self):
        return widgets.VBox(
            [
                widgets.HBox(
                    [self.w_parameters, self.w_method, self.w_transpose]),
                widgets.HBox([self.w_filter_key, self.w_filter_value]),
                widgets.VBox(
                    [
                        widgets.HBox(
                            [self.w_file_name, self.w_output_button,
                             self.w_overwrite]
                        ),
                        self.w_export_output,
                    ]
                ),
                widgets.interactive_output(
                    self.plot_callback,
                    {
                        'parameter': self.w_parameters,
                        'method': self.w_method,
                        'filter_key': self.w_filter_key,
                        'filter_value': self.w_filter_value,
                        'transpose': self.w_transpose,
                    },
                ),
            ]
        )

    def display(self):
        display(self.get_output())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pandasnotebooks']

package_data = \
{'': ['*']}

install_requires = \
['dbispipeline>=0.8.0,<0.9.0',
 'ipywidgets>=7.5.1,<8.0.0',
 'pandas>=1.1.1,<2.0.0']

setup_kwargs = {
    'name': 'pandasnotebooks',
    'version': '0.3.0',
    'description': 'Widgets to analyze, manipulate dataframes from dbispipelines.',
    'long_description': None,
    'author': 'Benjamin Murauer',
    'author_email': 'b.murauer@posteo.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



import os
import logging

import ipywidgets as widgets
from IPython.display import display


logger = logging.getLogger(__name__)


class LatexWidget:
    def __init__(
        self,
        df,
        git_commit_ids,
        row_ids=None,
        output_file=None,
        include_index=False,
        float_format='{:,.3f}',
        use_multirow=True,
        use_multicolumn=True,
        column_names=None,
        column_includes=None,
        column_formats=None,
        overwrite_output_file=False,
        highlight_max_per_column=False,
        escape=False,
        index_column_format='l',
    ):
        self.df = df
        self.output = widgets.Output()
        self.latex_out = widgets.Output(layout={'border': '1px solid grey'})
        self.w_file_log_output = widgets.Output()
        self.w_include_index = widgets.Checkbox(
            description='include index', value=include_index
        )
        self.w_index_column_format = widgets.Text(
            description='index col. format',
            value=index_column_format,
            disabled=not include_index,
        )
        self.w_include_index.observe(self._update_index_column_format)
        self.w_float_format = widgets.Text(
            description='float format', value=float_format
        )
        self.w_use_multirow = widgets.Checkbox(
            description='use multirows', value=use_multirow
        )
        self.w_use_multicolumn = widgets.Checkbox(
            description='use multicolumns', value=use_multicolumn
        )
        self.w_highlight_max_per_column = widgets.Checkbox(
            description='print the max value of each column in bold',
            value=highlight_max_per_column,
        )
        self.w_escape = widgets.Checkbox(description='Escape Latex',
                                         value=escape)
        self.w_output_file = widgets.Text(description='output file',
                                          value=output_file)
        self.w_draw_table_button = widgets.Button(
            description='draw latex table')
        self.w_write_file_button = widgets.Button(
            description='write to .tex file')
        self.w_overwrite_output_file = widgets.Checkbox(
            description='overwrite', value=overwrite_output_file
        )
        self.w_cols = {}
        self.output_widgets = []

        self.git_commit_ids = git_commit_ids
        self.row_ids = row_ids

        if column_names is None:
            column_names = df.columns
        if len(column_names) != len(df.columns):
            logger.error(
                "column_names doesn't have the correct length of "
                f'{len(df.columns)}'
            )
            return

        if column_includes is None:
            column_includes = [True] * len(column_names)
        if len(column_includes) != len(df.columns):
            logger.error(
                "column_includes doesn't have the correct length of "
                f'{len(df.columns)}'
            )
            return

        if column_formats is None:
            column_formats = ['c'] * len(column_names)
        if len(column_formats) != len(df.columns):
            logger.error(
                "column_formats doesn't have the correct length of "
                f'{len(df.columns)}'
            )
            return

        for i, column in enumerate(self.df.columns):
            self.w_cols[column] = {
                'name': widgets.Text(
                    value=str(column_names[i] if column_names[i] else column),
                    layout=widgets.Layout(width='140px'),
                ),
                'sort_up': widgets.Button(
                    icon='arrow-up',
                    tooltip=str(column),
                    layout=widgets.Layout(width='30px'),
                ),
                'sort_down': widgets.Button(
                    icon='arrow-down',
                    tooltip=str(column),
                    layout=widgets.Layout(width='30px'),
                ),
                'export': widgets.Checkbox(
                    value=column_includes[i],
                    layout=widgets.Layout(width='140px')
                ),
                'format': widgets.Text(
                    value=column_formats[i],
                    layout=widgets.Layout(width='30px')
                ),
            }
            self.output_widgets.append(
                widgets.HBox(
                    [
                        widgets.Label(
                            layout=widgets.Layout(min_width='140px'),
                            value=str(column)
                        ),
                        self.w_cols[column]['export'],
                        self.w_cols[column]['name'],
                        self.w_cols[column]['sort_up'],
                        self.w_cols[column]['sort_down'],
                        self.w_cols[column]['format'],
                    ]
                )
            )

            self.w_cols[column]['sort_up'].on_click(self._sort_up)
            self.w_cols[column]['sort_down'].on_click(self._sort_down)

        self.w_draw_table_button.on_click(self._draw_table)
        self.w_write_file_button.on_click(self._write_file)
        self._redraw()
        self.display()

    def _sort_up(self, button):
        cols = list(self.df.columns)
        old = cols.index(button.tooltip)
        if old == 0:
            return
        new = old - 1
        cols[new], cols[old] = cols[old], cols[new]
        self.output_widgets[new], self.output_widgets[old] = (
            self.output_widgets[old],
            self.output_widgets[new],
        )
        self.df = self.df[cols]
        self._redraw()

    def _sort_down(self, button):
        cols = list(self.df.columns)
        old = cols.index(button.tooltip)
        if old == len(cols) - 1:
            return
        new = old + 1
        cols[new], cols[old] = cols[old], cols[new]
        self.output_widgets[new], self.output_widgets[old] = (
            self.output_widgets[old],
            self.output_widgets[new],
        )
        self.df = self.df[cols]
        self._redraw()

    def _get_table_content(self):
        _columns = [
            c for c in self.df.columns if self.w_cols[c]['export'].value
        ]
        selected_names = [self.w_cols[c]['name'].value for c in
                          _columns]
        _formats = ''.join(
            [self.w_cols[c]['format'].value for c in _columns]
        )
        if self.w_include_index.value:
            _formats = self.w_index_column_format.value + _formats
        result = ''

        if self.git_commit_ids is not None:
            result += '% This table was made with results from commit: '
            result += str(self.git_commit_ids) + '\n'
        formatters = self._generate_formatters()

        if self.row_ids is not None:
            r_min, r_max = min(self.row_ids), max(self.row_ids)
            result += '% This table contains results from rows '
            f'{r_min} - {r_max}\n'

        result += self.df.to_latex(
            columns=_columns,
            header=selected_names,
            column_format=_formats,
            index=self.w_include_index.value,
            float_format=self.w_float_format.value.format,
            multirow=self.w_use_multirow.value,
            multicolumn=self.w_use_multicolumn.value,
            formatters=formatters,
            escape=self.w_escape.value,
        )
        return result

    def _generate_formatters(self):
        if self.w_highlight_max_per_column.value:
            formatters = []
            for column in self.df.columns:
                if self.df[column].dtype.kind in 'iuf':
                    # int, unsigned-int, float
                    formatters.append(self._generate_max_formatter(column))
                else:
                    formatters.append(None)
            return formatters
        return None

    def _generate_max_formatter(self, column):
        def formatter(value):
            raw_value = self.w_float_format.value.format(value)
            if value == self.df[column].max():
                return f'\\textbf{{ {raw_value} }} '
            return raw_value
        return formatter

    def _write_file(self, button=None):
        with self.w_file_log_output:
            self._draw_table()  # make sure the user sees what is written
            path = self.w_output_file.value
            if os.path.isfile(path) and not self.w_overwrite_output_file.value:
                logger.error(f'not overwriting existing file: {path}')
                return
            with open(path, 'w') as o_fh:
                o_fh.write(self._get_table_content())
                self.w_file_log_output.clear_output()
                print(f'content written to {path}')

    def _draw_table(self, button=None):
        with self.latex_out:
            self.latex_out.clear_output()
            print(self._get_table_content())

    def _redraw(self):
        with self.output:
            self.output.clear_output()
            display(widgets.VBox(self.output_widgets))
            self._draw_table()

    def _update_index_column_format(self, event):
        self.w_index_column_format.disabled = not self.w_include_index.value

    def display(self):
        display(self.get_output())

    def get_output(self):
        return widgets.VBox(
            [
                self.output,
                widgets.HBox(
                    [self.w_include_index, self.w_index_column_format]),
                self.w_use_multirow,
                self.w_use_multicolumn,
                self.w_highlight_max_per_column,
                self.w_escape,
                self.w_float_format,
                self.w_draw_table_button,
                self.latex_out,
                widgets.HBox(
                    [
                        self.w_output_file,
                        self.w_overwrite_output_file,
                        self.w_write_file_button,
                    ]
                ),
                self.w_file_log_output,
            ]
        )



import ipywidgets as widgets
from IPython.display import display
from dbispipeline.analytics import (
    rows_to_dataframe,
)
from dbispipeline.db import DbModel, DB

valid_input_filters = {}


class LoaderWidget:
    def __init__(
            self,
            row_ids=None,
            git_ids=None,
            project_name=None,
            plan_file=None,
            allow_dirty_rows=False,
            allow_multiple_git_ids=False,
            auto_open=True,
    ):
        layout = widgets.Layout(width='70%')
        s = {'description_width': '140px'}
        self.filter_widgets = {
            'id': widgets.Text(
                description='row ID(s)', value=row_ids, layout=layout,
                style=s),
            'git_commit_id': widgets.Text(
                description='git commit ID(s)', value=git_ids, layout=layout,
                style=s),
            'project_name': widgets.Text(
                description='project name(s)', value=project_name,
                layout=layout, style=s),
            'sourcefile': widgets.Text(
                description='plan file(s)', value=plan_file, layout=layout,
                style=s),
        }
        self.allow_dirty_rows = widgets.Checkbox(
                description='Allow dirty GIT rows', value=allow_dirty_rows)
        self.allow_multiple_git_ids = widgets.Checkbox(
                description='Allow multiple GIT commit ids',
                value=allow_multiple_git_ids)
        self.load_rows_button = widgets.Button(description="load")
        self.load_rows_button.on_click(self.load)
        if auto_open:
            display(
                widgets.VBox([
                    widgets.VBox(list(self.filter_widgets.values())),
                    self.allow_dirty_rows,
                    self.allow_multiple_git_ids,
                    self.load_rows_button,
                ], layout={'border': '1px solid grey'})
            )
        self.df = None

    def load(self, *args):
        session = DB.session()
        query = session.query(DbModel)
        for field, widget in self.filter_widgets.items():
            if widget.value:
                if ',' in widget.value:
                    query = query.filter(
                        getattr(DbModel, field).in_(widget.value.split(',')))
                else:
                    query = query.filter(
                        getattr(DbModel, field) == widget.value)
        self.df = rows_to_dataframe(
            query,
            allow_git_dirty_rows=self.allow_dirty_rows.value,
            allow_git_different_rows=self.allow_multiple_git_ids.value)
        print(f'df loaded with shape {self.df.shape}')
